@@ -49,6 +49,13 @@ sub new
                        attr => [Google::Checkout::XML::Constants::ITEM_CURRENCY,
                                 $currency_supported]);
   }
+  
+  if ($args{command}->get_reason)
+  {
+    $self->add_element(close => 1,
+                       name => Google::Checkout::XML::Constants::REASON,
+                       data => $args{command}->get_reason);
+  }
 
   return bless $self => $class;
 }
